function Common_WaitForSaveStart()
	local msgTable = 
	{
		lua_sys.Action_Ptr(), lua_sys.M_SavingGameStart_GetMsgTypeId(), function (msg)
			return true
		end,
	}
	
	coroutine.yield( msgTable )
end

function Common_WaitForSaveEnd()
	local msgTable = 
	{
		lua_sys.Action_Ptr(), lua_sys.M_SavingGameEnd_GetMsgTypeId(), function (msg)
			return true
		end,
	}
	
	coroutine.yield( msgTable )
end

function Common_WaitForDialogDone()
	local msgTable = 
	{
		lua_sys.Action_Ptr():getDialog():receiver(), lua_sys.M_DialogFinished_GetMsgTypeId(), function (msg)
			return true
		end,
	}

	coroutine.yield( msgTable )
end

function Common_WaitForTime(timetowait)
	
	local msgTable = 
	{
		lua_sys.Action_Ptr(), lua_sys.M_GameUpdate_GetMsgTypeId(), function (msg)
			
			timetowait = timetowait - msg.time;
			if timetowait <= 0 then
				return true
			else
				return false
			end
			
		end,
	}

	coroutine.yield( msgTable )
end
------------------------------------------------------------

function Common_ShowSaveLights(bool_show)
	local PREFIX = "furnace_"
	local objs = {}
	
	table.insert( objs, lua_sys.Action_Ptr():objectFind( PREFIX .. "top" ) )
	table.insert( objs, lua_sys.Action_Ptr():objectFind( PREFIX .. "bottom" ) )
	table.insert( objs, lua_sys.Action_Ptr():objectFind( PREFIX .. "small" ) )
	table.insert( objs, lua_sys.Action_Ptr():objectFind( PREFIX .. "side" ) )
	table.insert( objs, lua_sys.Action_Ptr():objectFind( PREFIX .. "side2" ) )
	
	for k,v in pairs(objs) do
		if not(v == nil) then
			v:setVisible(bool_show)
		end
	end
end

------------------------------------------------------------

function Common_HandleElevator(VARNAME_ELEVATOR_ACTIVATED)
	--gather all parts

	local obj
	local num
	local floorstops = {}
	--local sfxElevatorRun = lua_sys.Action_Ptr():makeSFX("audio/sfx/environment/elevator/elevator.wav")
	--sfxElevatorRun:PlayLoop()
	--sfxElevatorRun:Pause(true)

	--gather elevator stops points
	num = 1
	repeat
		local floorname = "FloorStop_0"..num
		obj = lua_sys.Action_Ptr():objectFind(floorname)
		if obj ~= nil then
			floorstops[num-1] = obj
			num = num + 1
		else
			break
		end
	until false

	if num <= 1 then
		--has no elevator stop points
		return
	end


	--gather all elevator object parts...for moving later
	local elevator_objs = {}
	local objs

	objs = lua_sys.Action_Ptr():objectGatherByLayerName("ELEVATOR FRONT")
	for i = 0, objs:size()-1 do
		table.insert( elevator_objs, objs[i] )
	end

	objs = lua_sys.Action_Ptr():objectGatherByLayerName("ELEVATOR GEAR")
	for i = 0, objs:size()-1 do
		table.insert( elevator_objs, objs[i] )
	end

	objs = lua_sys.Action_Ptr():objectGatherByLayerName("ELEVATOR WALL")
	for i = 0, objs:size()-1 do
		table.insert( elevator_objs, objs[i] )
	end

	--get player pos and find nearest elevator floor for the elevator to start there
	local player = lua_sys.Action_Ptr():playerGet()


	--get switches
	local elevator_switchup = lua_sys.Action_Ptr():objectFind("ElevatorSwitchUp")	--trigger
	local elevator_switchdown = lua_sys.Action_Ptr():objectFind("ElevatorSwitchDown")	--trigger
	local elevator_activator = lua_sys.Action_Ptr():objectFind("ElevatorEngine")		--trigger
	local elevator_activator_light = lua_sys.Action_Ptr():objectFind("elevator_engine_light")
	local elevator_lightdown = lua_sys.Action_Ptr():objectFind("elevator_light_down")
	local elevator_lightup = lua_sys.Action_Ptr():objectFind("elevator_light_up")
	local elevator_switch_lightup = lua_sys.Action_Ptr():objectFind("elevator_switch_light_up")
	local elevator_switch_lightdown = lua_sys.Action_Ptr():objectFind("elevator_switch_light_down")
	local elevator_trigger = lua_sys.Action_Ptr():objectFind("ElevatorTrigger")

	elevator_lightdown:setVisible( false )
	elevator_lightup:setVisible( false )
	elevator_switch_lightup:setVisible( false )
	elevator_switch_lightdown:setVisible( false )
	elevator_activator_light:setVisible( false )

	local nearestdist = 99999
	local nearestfloor = nil
	local nearestfloor_key = nil

	for key,val in pairs(floorstops) do
		local dist = math.abs( player:getRelPosY() - val:getRelPosY() )
		if dist < nearestdist then
			nearestdist = dist
			nearestfloor = val
			nearestfloor_key = key
		end
	end

	local dist = nearestfloor:getRelPosY() - floorstops[0]:getRelPosY()

	for key,x in pairs(elevator_objs) do
		x:setRelPos( x:getRelPosX(), x:getRelPosY() + dist )
	end

	local floor_to_goto_key = nearestfloor_key
	local floor_curpos = nearestfloor:getRelPosY()
	local floor_curmovedir = 0
	local elevator_enabled = false
	
	local elevator_light_flicker = false
	local elevator_flicker_flag = false
	local elevator_flicker_time = 1
	local elevator_flicker_count = 11

	--check if already activated
	
	if VARNAME_ELEVATOR_ACTIVATED=="" or lua_sys.Action_Ptr():getVarInt(VARNAME_ELEVATOR_ACTIVATED) ~= 0 then
		elevator_enabled = true
	end

	if elevator_enabled then
		elevator_lightup:setVisible( true )
		elevator_lightdown:setVisible( true )
		elevator_activator_light:setVisible( true )
	end


	local function moveToNextFloor()
		floor_to_goto_key = floor_to_goto_key + floor_curmovedir
		
		if floor_to_goto_key > table.getn( floorstops ) or floor_to_goto_key < 0 then
			floor_to_goto_key = floor_to_goto_key - floor_curmovedir
			floor_curmovedir = 0
			elevator_switch_lightdown:setVisible( false )
			elevator_switch_lightup:setVisible( false )	
			--floor_curmovedir = floor_curmovedir * -1
			--floor_to_goto_key = floor_to_goto_key + floor_curmovedir*2
				
		else
			lua_sys.Action_Ptr():playSFX("audio/sfx/environment/elevator/elevator_start.wav")
			--sfxElevatorRun:Pause(false)
			lua_sys.Action_Ptr():playSFX("audio/sfx/environment/elevator/elevator.wav")
			lua_sys.Action_Ptr():ElevatorMove(floor_curmovedir)
		end
		
	end

	local function isPlayerCloseToElevatorSwitches( obj )
		local dist = ( (player:absPosX()) - (obj:absPosX() + obj:sizeX()/2))
		return  dist > -40 and dist < 40	
	end
		
	local function canMove()
		local flrtogoto =  floor_to_goto_key 
		local flrcurmove=0
		if elevator_switch_lightdown and elevator_switch_lightup and player then
			if elevator_enabled then
				if isPlayerCloseToElevatorSwitches(elevator_switch_lightdown) then
					flrcurmove=-1
					flrtogoto = flrtogoto + flrcurmove
				--[[if flrtogoto > table.getn( floorstops ) or flrtogoto < 0 then
					player.canAttack_=true
					player.playerOnTrigger_=false
					
				else
					player.canAttack_=false
					player.playerOnTrigger_=true			
				end[]]--
				elseif isPlayerCloseToElevatorSwitches(elevator_switch_lightup) then
					flrcurmove=1
					flrtogoto = flrtogoto + flrcurmove
				--[[if flrtogoto > table.getn( floorstops ) or flrtogoto < 0 then
					player.canAttack_=true
					player.playerOnTrigger_=false
					
				else
					player.canAttack_=false
					player.playerOnTrigger_=true			
				end]]--
				
				end
			end
		end
	end
	
	local function elevatorLightFlicker(tickTime)
		
		if(elevator_light_flicker and elevator_flicker_count > 0)
		then
			elevator_flicker_time = elevator_flicker_time - tickTime
			if(elevator_flicker_time <= 0)
			then
				elevator_flicker_count = elevator_flicker_count - 1;
				if(elevator_flicker_count == 10)
				then
					elevator_flicker_time = 0.6
				elseif(elevator_flicker_count == 2)
				then
					elevator_flicker_time = 0.6
				elseif(elevator_flicker_count == 1)
				then
					elevator_flicker_time = 0.8
				else
					elevator_flicker_time = 0.1
				end
				elevator_flicker_flag = not elevator_flicker_flag
				elevator_lightup:setVisible( elevator_flicker_flag )
				elevator_lightdown:setVisible( elevator_flicker_flag )
				elevator_activator_light:setVisible( elevator_flicker_flag )
			end
		end
	end

	while true do
		
		local msgTable = 
		{
			lua_sys.Action_Ptr(), lua_sys.M_GameUpdate_GetMsgTypeId(), function (msg)
				
				if floor_curmovedir ~= 0 then
				
					local target = floorstops[floor_to_goto_key]
					
					local dif = target:getRelPosY() - floor_curpos
					local dist = math.abs( dif )
					
					if dist < 1 then
						--snap to pos
						for key,x in pairs(elevator_objs) do
							x:setRelPos( x:getRelPosX(), x:getRelPosY() + dif )
						end
						floor_curpos = floor_curpos + dif
						
						if dif> 0 then
							player:setVel( player:getVelX(), player:getVelY()+dif )
						end					
						
						floor_curmovedir = 0
						--sfxElevatorRun:Pause(true)
						lua_sys.Action_Ptr():playSFX("audio/sfx/environment/elevator/elevator_end.wav")
						
						elevator_switch_lightdown:setVisible( false )
						elevator_switch_lightup:setVisible( false )
						
						lua_sys.Action_Ptr():ElevatorMove(0)
						
						if elevator_enabled then
						
							--move to next floor
							--moveToNextFloor()
						
						end
						
					else
						--move to pos
						
						local dir
						if dif<0 then
							dir = -1
						else
							dir = 1
						end
						
						local speed = 1
						if dist > 100 then
							speed = 300
						else
							local t = 1 - (dist / 100)
							speed = 300-300 * t
							if speed < 1 then
								speed = 1
							end
						end
						
						local finalmovespeed = dir * speed
						
						local moveamount = finalmovespeed * msg.time
						
						for key,x in pairs(elevator_objs) do
							x:setRelPos( x:getRelPosX(), x:getRelPosY() + moveamount )
						end			
						
						if finalmovespeed> 0 then
							player:setVel( player:getVelX(), player:getVelY()+finalmovespeed )
						end
						
						floor_curpos = floor_curpos + moveamount
					end
				end
				
				canMove()
				elevatorLightFlicker(msg.time)
				
				return true
			end,
			---------
			--[[
			lua_sys.Action_Ptr(), lua_sys.M_PlayerOnTrigger_GetMsgTypeId(), function (msg)
				if floor_curmovedir == 0 then
					if elevator_enabled then
						if isPlayerCloseToElevatorSwitches(elevator_switch_lightdown) then		
							if msg.triggerName == "ElevatorSwitchDown" then
								if player.isPushingButton_  then	
									floor_curmovedir = -1
									elevator_switch_lightdown:setVisible( true )
									moveToNextFloor()
							
								end
							end
						end
						
						if isPlayerCloseToElevatorSwitches(elevator_switch_lightup) then	
							if msg.triggerName == "ElevatorSwitchUp" then
								if player.isPushingButton_  then		
									floor_curmovedir = 1
									elevator_switch_lightup:setVisible( true )
									moveToNextFloor()
								
								end
							end
						end
					end
					
				end
			
			
				return true
			end,]]--
			---------
			lua_sys.Action_Ptr(), lua_sys.M_PlayerOffTrigger_GetMsgTypeId(), function (msg)

				return true
			end,
			---------
			lua_sys.Action_Ptr(), lua_sys.M_PlayerInTrigger_GetMsgTypeId(), function (msg)

				return true
			end,		
			---------
			lua_sys.Action_Ptr(), lua_sys.M_PlayerPushingElevatorButton_GetMsgTypeId(), function (msg)
				if floor_curmovedir == 0 then
					if elevator_enabled then
						if isPlayerCloseToElevatorSwitches(elevator_switch_lightdown) then	
							floor_curmovedir = -1
							elevator_switch_lightdown:setVisible( true )
							moveToNextFloor()
													
						elseif isPlayerCloseToElevatorSwitches(elevator_switch_lightup) then
							floor_curmovedir = 1
							elevator_switch_lightup:setVisible( true )
							moveToNextFloor()
						
						end
					end	
				end
				
				return true
			end,
			------------
			lua_sys.Action_Ptr(), lua_sys.M_PlayerUsedElectricBurst_GetMsgTypeId(), function (msg)
			
				local dx = player:absPosX() - elevator_activator:absPosX()
				local dy = player:absPosY() - elevator_activator:absPosY()
				
				local dist = dx*dx+dy*dy
				
				if dist < 50*50 then
					--if msg.triggerName == "ElevatorEngine" then
					if not elevator_enabled then
					
						if lua_sys.Action_Ptr():playerHasElectric() then
							lua_sys.Action_Ptr():playSFX("audio/sfx/environment/elevator/elevator_activate.wav")
							elevator_enabled = true
							elevator_light_flicker = true
							--elevator_lightup:setVisible( true )
							--elevator_lightdown:setVisible( true )
							--elevator_activator_light:setVisible( true )
							if VARNAME_ELEVATOR_ACTIVATED ~= "" then
								lua_sys.Action_Ptr():setVarInt(VARNAME_ELEVATOR_ACTIVATED, 1 );
							end
						end
						
					end
				end
				return true
			end,
			---------
			
			
		}
		coroutine.yield( msgTable )	
		
	end
	
end
	
------------------------------------------------------------
function Common_StoryItem()
	
	lua_sys.Action_Ptr():playSFX("audio/sfx/environment/item/story_item.wav")
end

------------------------------------------------------------

function Common_GiveFuseOnce(uniquename, fusetype)
	--if lua_sys.Action_Ptr():getVarInt(uniquename) == 0 then
	--	lua_sys.Action_Ptr():setVarInt(uniquename, 1 );
		lua_sys.Action_Ptr():giveFuse(fusetype);
	--end 
end
